/***********************************************************************************
Complex.java - classe para manusear nmeros complexos.

Direitos Autorais Reservados (c) 2006 Embrapa Informtica Agropecuria

Este programa  software livre; voc pode redistribu-lo e/ou modific-lo sob os
termos da Licena Pblica Geral GNU conforme publicada pela Free Software
Foundation; tanto a verso 2 da Licena, como (a seu critrio) qualquer verso
posterior.

Este programa  distribudo na expectativa de que seja til, porm, SEM NENHUMA
GARANTIA; nem mesmo a garantia implcita de COMERCIABILIDADE OU ADEQUAO A UMA
FINALIDADE ESPECFICA. Consulte a Licena Pblica Geral do GNU para mais detalhes.

Voc deve ter recebido uma cpia da Licena Pblica Geral do GNU junto com este
programa; se no, escreva para a Free Software Foundation, Inc., no endereo 59
Temple Street, Suite 330, Boston, MA 02111-1307 USA.

**********************************************************************************

Parte integrante do Comunicado Tcnico XXXXXXX.

Descrio:

	Classe usada para manusear nmeros complexos.

  Observaes do implementador:

  Data type for complex numbers.
 
   The data type is "immutable" so once you create and initialize
   a complex object, you cannot change its value. The "final"
   keyword when declaring re and im enforces this rule, making it
   a compile-time error to change the .re or .im fields after
   they've been initialized.
 
   % java Complex
   a = a = 5.0 + 6.0i
   b = -3.0 + 4.0i
   c = -39.0 + 2.0i
   d = -39.0 + -2.0i
   e = 5.0

Parmetros de entrada:
  
  No se aplica.

Sada:

  No se aplica.

Desenvolvido por:

    Annimo.

Informaes do CVS:
       $Source$:
       $Revision$:
       $Date$:

***********************************************************************************/

public class Complex {
	public  double re;   // the real part
  public  double im;   // the imaginary part
	
  // create a new object with the given real and imaginary parts
  public Complex(double real, double imag) {
		this.re = real;
    this.im = imag;
  }

  public Complex() {
    this.re = 0;
    this.im = 0;
  }
    
  // return a string representation of the invoking object
  public String toString()  { return re + " + " + im + "i"; }

  // return |this|
  public double abs() { return Math.sqrt(re*re + im*im);  }

  // return a new object whose value is (this + b)
  public Complex plus(Complex b) { 
		Complex a = this;             // invoking object
    double real = a.re + b.re;
    double imag = a.im + b.im;
    Complex sum = new Complex(real, imag);
    return sum;
	}

	// return a new object whose value is (this - b)
	public Complex minus(Complex b) { 
		Complex a = this;   
    double real = a.re - b.re;
    double imag = a.im - b.im;
    Complex diff = new Complex(real, imag);
    return diff;
  }

  // return a new object whose value is (this * b)
  public Complex times(Complex b) {
		Complex a = this;
    double real = a.re * b.re - a.im * b.im;
    double imag = a.re * b.im + a.im * b.re;
    Complex prod = new Complex(real, imag);
    return prod;
	}

	// return a new object whose value is (this * alpha)
  public Complex times(double alpha) {
    return new Complex(alpha * re, alpha * im);
  }

	public Complex divide(double b){
		Complex a = this;             // invoking object
    double real, imag;

    if (b != 0.0) {
      real = a.re/b;
      imag = a.im/b;
    } else {
      String str = "Complex::divide(double b) -> division by zero.";
      System.out.println(str);
		  return new Complex(0.0, 0.0);
    }

		return new Complex(real, imag);
	}


	public void divide(Complex b){
		Complex a = this;             // invoking object
    double real = a.re*b.re+a.im*b.im;
    double imag = a.im*b.re-a.re*b.im;
		double denominador = b.re*b.re+b.im*b.im;

    if (denominador != 0.0) {
  		a.re = real/denominador;
	  	a.im = imag/denominador;
    } else {
      String str = "Complex::divide(Complex b) -> division by zero.";
      System.out.println(str);
  		a.re = real/1;
	  	a.im = imag/1;
    }
	}
  // return a new object whose value is the conjugate of this
  public Complex conjugate() { return new Complex(re, -im); }

}