/***********************************************************************************
FiltroDifusaoComplexaNaoLinear.java - realce de imagens por difuso complexa no
                                      linear ou anisotrpica.

Direitos Autorais Reservados (c) 2006 Embrapa Informtica Agropecuria

Este programa  software livre; voc pode redistribu-lo e/ou modific-lo sob os
termos da Licena Pblica Geral GNU conforme publicada pela Free Software
Foundation; tanto a verso 2 da Licena, como (a seu critrio) qualquer verso
posterior.

Este programa  distribudo na expectativa de que seja til, porm, SEM NENHUMA
GARANTIA; nem mesmo a garantia implcita de COMERCIABILIDADE OU ADEQUAO A UMA
FINALIDADE ESPECFICA. Consulte a Licena Pblica Geral do GNU para mais detalhes.

Voc deve ter recebido uma cpia da Licena Pblica Geral do GNU junto com este
programa; se no, escreva para a Free Software Foundation, Inc., no endereo 59
Temple Street, Suite 330, Boston, MA 02111-1307 USA.

**********************************************************************************

Parte integrante do Comunicado Tcnico XXXXXXX.

Descrio:

  Implementa o filtro de realce de imagens e eliminao de rudos usando um filtro
  no linear, ou anisotrpico, de difuso complexa.

  A implementao do filtro tem por referncia o artigo:

  GILBOA, G.; SOCHE, N.; ZEEVI, Y. Y. Image enhancement and denoising by complex 
    diffusion processes. IEEE Transactions on Pattern Analysis and Machine 
    Intelligence. 26(8):1020-1036, 2004.

  Este programa depende das classes Complex.java e MatrizComplexa.java, encontrados
  neste diretrio.

Parmetros de entrada:

  1. Nome do arquivo imagem.
  2. Nmero de iteraes. Equivale ao parmetro 'tempo' na equao (XXX) do 
     Comunicado Tcnico.
  3. 'limiar'. Para incio, pode usar limiar = 5.

Sada:

  Painel com a imagem original e imagem filtrada. A imagem filtrada pode ser a 
  parte real ou imaginria da imagem complexa. Nesse caso, a preferncia  pela
  parte real. A parte complexa equivale ao filtro linear implementado no programa
  FiltroDifusaoComplexa.java, do Comunicado Tcnico XXXXXXXX.

Desenvolvido por:

    Jos Iguelmar Miranda & Joo Camargo Neto.

Informaes do CVS:
       $Source$:
       $Revision$:
       $Date$:

***********************************************************************************/

// pacotes genricos
import java.io.File;
import java.io.IOException;

// pacotes do AWT
import java.awt.image.WritableRaster;
import java.awt.image.BufferedImage;
import java.awt.image.Raster;
import java.awt.GridLayout;

// pacote Swing para interface grfica
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.ImageIcon;
import javax.swing.JScrollPane;

// leitura em modo imediato - para J2SE 1.4+
import javax.imageio.ImageIO;
	
public class FiltroDifusaoComplexaNaoLinear extends JFrame {

	// ngulo de fase do coeficiente de difuso
	double theta = Math.PI/1000;			

  public static void main(String args[]) {
		int iteracao = 0, limiar = 5;

		// Certifica a passagem de parmetros
    if (args.length != 3) {
			String msga = "Uso: java -cp . FiltroDifusaoComplexaNaoLinear ";
			String msgb = "<imagem> <interacoes> <limiar>";
			System.out.println(msga + msgb);
      System.exit(0);
    }

		// Mostra JFrame decorado pelo Swing
		JFrame.setDefaultLookAndFeelDecorated(true);

		long eq_time = System.currentTimeMillis();

		try {
        iteracao = Integer.parseInt(args[1]);
    } catch (NumberFormatException e) {
				String st = "Valor do parametro 'interacoes' invalido";
        System.out.println(st);
        System.exit(0);
    }

		try {
        limiar = Integer.parseInt(args[2]);
    } catch (NumberFormatException e) {
				String st = "Valor do parametro 'limiar' invalido";
        System.out.println(st);
        System.exit(0);
    }

		System.out.println("\nFiltro de difusao complexa anisotropico - Parametros:\n");
		String st = "\t#interacoes: ";
	  System.out.println(st + iteracao);
		st = "\n\tlimiar: ";
	  System.out.println(st + limiar + "\n");

		FiltroDifusaoComplexaNaoLinear c = 
			new FiltroDifusaoComplexaNaoLinear(args[0], iteracao, limiar); 

    eq_time = System.currentTimeMillis() - eq_time;
		String msg = "FiltroComplexo: tempo de execucao ";
    System.out.println(msg + eq_time + " milisseg.");

		// Encerra a aplicao clicando no "close"
		c.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		c.setVisible(true);
	}
  
	public FiltroDifusaoComplexaNaoLinear(String aFile, int iteracao, int limiar) {
		
		// Define objeto BufferedImage para encapsular a imagem
    BufferedImage src = null, dest = null, dest1 = null;
		int w, h, tipo, limiarR, pixels[][], bordas[][];
		JLabel img1, img2;

		// Certifica que a imagem existe
    File file = new File(aFile);
    try {
        src = ImageIO.read(file);
    } catch(Exception e) {
        System.out.println("Imagem '" + aFile + "' nao existe.");
        System.exit(0);
    }

    // Atribui nome ao frame
    setTitle("Difuso complexa anisotrpica: " + file.getName());

		w = src.getWidth();
		h = src.getHeight();
		
    // Mtodo de preservao de rampa (difuso complexa no linear).
		pixels = lePixels(src);
		bordas = difusaoRampa(pixels, iteracao, limiar);
		dest = criaImagem(bordas);
	
		// Cria gridLayout de 1 x 2
    getContentPane().setLayout(new GridLayout(1, 2));
		img1 = new JLabel(new ImageIcon(src));
		img2 = new JLabel(new ImageIcon(dest));
		getContentPane().add(new JScrollPane(img1));
		getContentPane().add(new JScrollPane(img2));
		setSize(2*w, h);
	}

	private int[][] lePixels(BufferedImage src) {
		int w, h, pixels[][] = null;

		w = src.getWidth();
		h = src.getHeight();
		pixels = new int[w][h];
    Raster srcR = src.getRaster(); 

		for (int i = 0; i < h; i++)
			for (int j = 0; j < w; j++) {
				pixels[j][i] = srcR.getSample(j, i, 0);
			}

		return pixels;
	}

	private int[][] difusaoRampa(int[][] in, int interacoes, int limiar){
		int h, w, bfi[][];
		double dt, theta;
		double[][] buferMatriz;
		Complex j;
		MatrizComplexa cN, cS, cL, cO, imaginarioTemp;
		
	  w = in.length;
		h = in[0].length;  
		bfi  = new int[w][h];
	  dt = 0.2;
		// pequeno ngulo de fase
		theta = Math.PI/1000;
		// j = sqrt(-1); 
		// j*theta => exp(a + bi) = (exp(a)*cos(b), exp(a)*sin(b))
		j = new Complex(Math.exp(0.0)*Math.cos(theta), 
											 Math.exp(0.0)*Math.sin(theta));
		
		//Criao dos objetos MatrizComplexa a partir da imagen
		MatrizComplexa m1ComplexaJ = new MatrizComplexa(in);
		MatrizComplexa m1ComplexaN = new MatrizComplexa(in);
		MatrizComplexa m1ComplexaS = new MatrizComplexa(in);
		MatrizComplexa m1ComplexaL = new MatrizComplexa(in);
		MatrizComplexa m1ComplexaO = new MatrizComplexa(in);

		cN = new MatrizComplexa(w, h);
		cS = new MatrizComplexa(w, h);
		cL = new MatrizComplexa(w, h);
		cO = new MatrizComplexa(w, h);
		imaginarioTemp = new MatrizComplexa(w, h);	

		System.out.println("Processando...");
		for (int i = 0; i < interacoes; i++){		

			m1ComplexaN.gradiente(1);
			m1ComplexaS.gradiente(2);
			m1ComplexaL.gradiente(3);
			m1ComplexaO.gradiente(4);

			System.out.print(".");

			// Inicializa as matrizes com o valor o "j" para cada elemento
			// Cn = exp(j*theta)./(1+(imag(In)/(k*theta)).^2);
			// k = limiar
			imaginarioTemp.retornaParteImaginaria(m1ComplexaN);	
			imaginarioTemp.divide(limiar*theta);
			imaginarioTemp.elevaQuadrado();
			imaginarioTemp.somaElemento(1.0);
			cN.divideElementoPorMatriz(j,imaginarioTemp);

			// Cs = exp(j*theta)./(1+(imag(Is)/(k*theta)).^2);
			// k = limiar
			imaginarioTemp.retornaParteImaginaria(m1ComplexaS);	
			imaginarioTemp.divide(limiar*theta);
			imaginarioTemp.elevaQuadrado();
			imaginarioTemp.somaElemento(1.0);
			cS.divideElementoPorMatriz(j,imaginarioTemp);

			// Ce = exp(j*theta)./(1+(imag(Ie)/(k*theta)).^2);
			// k = limiar
			imaginarioTemp.retornaParteImaginaria(m1ComplexaL);	
			imaginarioTemp.divide(limiar*theta);
			imaginarioTemp.elevaQuadrado();
			imaginarioTemp.somaElemento(1.0);
			cL.divideElementoPorMatriz(j,imaginarioTemp);

			// Cw = exp(j*theta)./(1+(imag(Iw)/(k*theta)).^2);
			// k = limiar
			imaginarioTemp.retornaParteImaginaria(m1ComplexaO);	
			imaginarioTemp.divide(limiar*theta);
			imaginarioTemp.elevaQuadrado();
			imaginarioTemp.somaElemento(1.0);
			cO.divideElementoPorMatriz(j,imaginarioTemp);

			// J = J + dt*(Cn.*In + Cs.*Is + Ce.*Ie + Cw.*Iw);
			m1ComplexaN.multiplica(cN);
			m1ComplexaS.multiplica(cS);
			m1ComplexaL.multiplica(cL);
			m1ComplexaO.multiplica(cO);
			
			m1ComplexaN.soma(m1ComplexaS);
			m1ComplexaO.soma(m1ComplexaL);
			m1ComplexaN.soma(m1ComplexaO);
			m1ComplexaN.multiplica(dt);
			m1ComplexaJ.soma(m1ComplexaN);
			
			m1ComplexaN.copia(m1ComplexaJ);
			m1ComplexaS.copia(m1ComplexaJ);
			m1ComplexaL.copia(m1ComplexaJ);
			m1ComplexaO.copia(m1ComplexaJ);			
		}

    // Para retornar a parte imaginria, comentar o cdigo abaixo e
    // descomentar o da parte imaginria
    // Retorna a parte real da imagem complexa
		buferMatriz = m1ComplexaJ.retornaParteReal();
		for(int lin = 0; lin < h; lin++){
			for(int col = 0; col < w; col++){
				bfi[col][lin] = (int) Math.floor(buferMatriz[col][lin]);      		 
			} 
		}

    // Retorna a parte imaginria da imagem complexa
    /*
		buferMatriz = m1ComplexaJ.retornaParteImaginaria();
		for(int lin = 0; lin < h; lin++){
			for(int col = 0; col < w; col++){
				bfi[col][lin] = (int) (128.0 + buferMatriz[col][lin]/theta);      		 
			} 
		}
    */

		// Libera rea de armazenamento.
		m1ComplexaN = null;
		m1ComplexaS = null;
		m1ComplexaL = null;
		m1ComplexaO = null;
		m1ComplexaJ = null;

		System.out.println("\n");
		return bfi;
	}

	private BufferedImage criaImagem(int[][] in) {
		int w, h, tipo, pixels[], ind = 0;
		BufferedImage dest = null;

	  w = in.length;
		h = in[0].length;
		pixels = new int[w*h];
    tipo = BufferedImage.TYPE_BYTE_GRAY;
    dest = new BufferedImage(w, h, tipo);
		WritableRaster destWR = dest.getRaster();

		for(int lin = 0; lin < h; lin++)
			for(int col = 0; col < w; col++)	
				pixels[ind++] = in[col][lin];

		destWR.setPixels(0, 0, w, h, pixels);
		return dest;		
	}
}