/***********************************************************************************
MatrizComplexa.java - classe para matrizes complexas.

Direitos Autorais Reservados (c) 2006 Embrapa Informtica Agropecuria

Este programa  software livre; voc pode redistribu-lo e/ou modific-lo sob os
termos da Licena Pblica Geral GNU conforme publicada pela Free Software
Foundation; tanto a verso 2 da Licena, como (a seu critrio) qualquer verso
posterior.

Este programa  distribudo na expectativa de que seja til, porm, SEM NENHUMA
GARANTIA; nem mesmo a garantia implcita de COMERCIABILIDADE OU ADEQUAO A UMA
FINALIDADE ESPECFICA. Consulte a Licena Pblica Geral do GNU para mais detalhes.

Voc deve ter recebido uma cpia da Licena Pblica Geral do GNU junto com este
programa; se no, escreva para a Free Software Foundation, Inc., no endereo 59
Temple Street, Suite 330, Boston, MA 02111-1307 USA.

**********************************************************************************

Parte integrante do Comunicado Tcnico XXXXXXX.

Descrio:

  Classe para manipular matrizes complexas.

Parmetros de entrada:

  No se aplica.

Sada:

  No se aplica.

Desenvolvido por:

    Jos Iguelmar Miranda & Joo Camargo Neto.

Informaes do CVS:
       $Source$:
       $Revision$:
       $Date$:

***********************************************************************************/

// pacote genrico
import java.text.DecimalFormat;

public class MatrizComplexa {

	// ATRIBUTOS
	public Complex[][] MComplexa;
	private int col, lin;

	// CONSTRUTORES DA CLASSE
	public MatrizComplexa(int c, int l){
		this.col = c;
		this.lin = l;
		this.MComplexa = new Complex[col][lin];
			for(int j = 0; j < lin; j++){
				for(int i = 0; i < col; i++){
					this.MComplexa[i][j] = new Complex(0.0, 0.0);
        }           
      }
  }
	
	// Transforma uma matriz de inteiro para uma matriz com valores complexo
	// onde o valor inteiro  a parte real e zero como valor imaginrio;
	public MatrizComplexa(int[][] a){
		col = a.length;
		lin = a[0].length;
		this.MComplexa = new Complex[col][lin];
			for(int j = 0; j < lin; j++){
				for(int i = 0; i < col; i++){              	
					this.MComplexa[i][j] = new Complex((double) a[i][j], 0.0);
				}              
      }
  }

	// MTODOS PBLICOS
	public int retornaNumeroLinhas(){
		return lin;
	}
	
	public int retornaNumeroColunas(){
		return col;
	}

	public Complex[][] retornaMatriz(){
		return MComplexa;
	}

	public void subtrai(MatrizComplexa b){
		double 	
      fatorI,  	        // fator imaginrio
		  fatorR;						// fator real
		
    for(int j = 0; j < lin; j++){
     	for(int i = 0; i < col; i++){
     		fatorR = this.MComplexa[i][j].re - b.MComplexa[i][j].re;
     		fatorI = this.MComplexa[i][j].im - b.MComplexa[i][j].im;  
     		this.MComplexa[i][j].re = fatorR;
     		this.MComplexa[i][j].im = fatorI;   	
     	}              
    }
	}
	
	public void soma(MatrizComplexa b){
		double 	
      fatorI,  	      // fator imaginrio
			fatorR;		      // fator real

    for(int j = 0; j < lin; j++){
     	for(int i = 0; i < col; i++){
     		fatorR = this.MComplexa[i][j].re + b.MComplexa[i][j].re;
     		fatorI = this.MComplexa[i][j].im + b.MComplexa[i][j].im;
     		this.MComplexa[i][j].re = fatorR;
     		this.MComplexa[i][j].im = fatorI;       	
     	}              
    }
  }

	public void somaElemento(Complex b){
		double 	
      fatorI,  	      // fator imaginrio
			fatorR;		      // fator real

    for(int j = 0; j < lin; j++){
     	for(int i = 0; i < col; i++){
     		fatorR = this.MComplexa[i][j].re + b.re;
     		fatorI = this.MComplexa[i][j].im + b.im;
     		this.MComplexa[i][j].re = fatorR;
     		this.MComplexa[i][j].im = fatorI;       	
     	}              
    }
  }

	public void atribuiElemento(Complex b){
		
    for(int j = 0; j < lin; j++){
     	for(int i = 0; i < col; i++){	
     		this.MComplexa[i][j].re = b.re;
     		this.MComplexa[i][j].im = b.im;       	
     	}              
    }
  }

	public void somaElemento(double b){
		double 	
      fatorI,  	//	fator imaginrio
			fatorR;		// 	fator real

    for(int j = 0; j < lin; j++){
     	for(int i = 0; i < col; i++){
     		fatorR = this.MComplexa[i][j].re + b;
     		fatorI = this.MComplexa[i][j].im + b;
     		this.MComplexa[i][j].re = fatorR;
     		this.MComplexa[i][j].im = fatorI;       	
     	}              
    }
  }
	
	public void multiplica(MatrizComplexa b){
		double 	
      fatorI,  	//	fator imaginrio
			fatorR;		// 	fator real
		
    for(int j = 0; j < lin; j++){
			for(int i = 0; i < col; i++){ 
     		fatorR = this.MComplexa[i][j].re* b.MComplexa[i][j].re - 
		  						this.MComplexa[i][j].im* b.MComplexa[i][j].im;
       	fatorI = this.MComplexa[i][j].re* b.MComplexa[i][j].im + 
	  							this.MComplexa[i][j].im* b.MComplexa[i][j].re;
        this.MComplexa[i][j].re = fatorR; 
        this.MComplexa[i][j].im = fatorI; 
      }              
    }
  }
	
	public void multiplica(double b){
		double 	
      fatorI,  	//	fator imaginrio
			fatorR;		// 	fator real

    for(int j = 0; j < lin; j++){
     	for(int i = 0; i < col; i++){
     		fatorR = this.MComplexa[i][j].re* b; 
     		fatorI = this.MComplexa[i][j].im* b; 
     		this.MComplexa[i][j].re = fatorR;
     		this.MComplexa[i][j].im = fatorI;
     	}              
    }
  }
	
	public void multiplica(Complex b){
		double 	
      fatorI,  	//	fator imaginrio
			fatorR;		// 	fator real

    for(int j = 0; j < lin; j++){
     	for(int i = 0; i < col; i++){ 
     		fatorR = this.MComplexa[i][j].re* b.re - this.MComplexa[i][j].im* b.im;
     		fatorI = this.MComplexa[i][j].re* b.im + this.MComplexa[i][j].im* b.re;       		
     		this.MComplexa[i][j].re = fatorR;
    		this.MComplexa[i][j].im = fatorI;
     	}
    }
  }
	
	public double[][] abs(){		
		double[][] absVec = new double[col][lin];
		 
    for(int j = 0; j < lin; j++)
     	for(int i = 0; i < col; i++)
				absVec[i][j] = this.MComplexa[i][j].abs();

		return absVec;
  }

	public void divide(double b){
		double 	
      fatorI,  	//	fator imaginrio
			fatorR;		// 	fator real

    for(int j = 0; j < lin; j++){
     	for(int i = 0; i < col; i++){
     		fatorR = this.MComplexa[i][j].re/ b; 
     		fatorI = this.MComplexa[i][j].im/ b; 
     		this.MComplexa[i][j].re = fatorR;
     		this.MComplexa[i][j].im = fatorI;
     	}              
    }
  }

	public void divideElemento(MatrizComplexa b){
		double 	
      fatorI,  	//	fator imaginrio
			fatorR;		// 	fator real

    for(int j = 0; j < lin; j++){
     	for(int i = 0; i < col; i++){
				//this.MComplexa[i][j].divide(b.MComplexa[i][j]);
				fatorR = this.MComplexa[i][j].re/b.MComplexa[i][j].re; 
     		fatorI = this.MComplexa[i][j].im/b.MComplexa[i][j].im; 
     		this.MComplexa[i][j].re = fatorR;
     		this.MComplexa[i][j].im = fatorI;
     	}              
    }
  }

  public void divideElementoPorMatriz(Complex c,MatrizComplexa b){
		for(int j = 0; j < lin; j++){
     	for(int i = 0; i < col; i++){              	               	
     		this.MComplexa[i][j].re = c.re/b.MComplexa[i][j].im;
     		this.MComplexa[i][j].im = c.im/b.MComplexa[i][j].im;               	
     	}              
    }
	}

	public void copia(MatrizComplexa b){
    for(int j = 0; j < lin; j++){
     	for(int i = 0; i < col; i++){              	               	
     		this.MComplexa[i][j].re = b.MComplexa[i][j].re;
     		this.MComplexa[i][j].im = b.MComplexa[i][j].im;               	
     	}              
    }
	}
	
	public double[][] retornaParteReal(){
		double[][] mcReal;
		
		Complex[][] a = this.MComplexa;
		mcReal = new double[col][lin];
    for(int j = 0; j < lin; j++){
     	for(int i = 0; i < col; i++){              	               	
     		mcReal[i][j] = a[i][j].re;
     	}              
    }
    return mcReal;
  }
	
	public double[][] retornaParteImaginaria(){
		double[][] mcImaginario;
		
		mcImaginario = new double[col][lin];
    for(int j = 0; j < lin; j++){
     	for(int i = 0; i < col; i++){              	               	
     		mcImaginario[i][j] = this.MComplexa[i][j].im;
     	}              
    }
    return mcImaginario;
  }
	
	public void retornaParteImaginaria(MatrizComplexa b){
		
    for(int j = 0; j < lin; j++){
     	for(int i = 0; i < col; i++){              	               	
     		this.MComplexa[i][j].re = 0.0;
     		this.MComplexa[i][j].im = b.MComplexa[i][j].im;
     	}              
    }
  }
	
	public void elevaQuadrado(){
		double 	
      fatorI,  	//	fator imaginrio
			fatorR;		// 	fator real

    for(int j = 0; j < lin; j++){
     	for(int i = 0; i < col; i++){
     		fatorR = this.MComplexa[i][j].re*this.MComplexa[i][j].re; 
     		fatorI = this.MComplexa[i][j].im*this.MComplexa[i][j].im; 
     		this.MComplexa[i][j].re = fatorR;
     		this.MComplexa[i][j].im = fatorI;
     	}              
    }
  }

	public void inverteElemento(){
		double 	
      fatorI,  	//	fator imaginrio
			fatorR,		// 	fator real
			fatorR2,
			fatorI2;

    for(int j = 0; j < lin; j++){
     	for(int i = 0; i < col; i++){
     		fatorR = this.MComplexa[i][j].re; 
     		fatorI = 1.0/this.MComplexa[i][j].im; 
				fatorR2 = fatorR*fatorR;
				fatorI2 = fatorI*fatorI;
     		this.MComplexa[i][j].re = fatorR/(fatorR2+fatorI2);
     		this.MComplexa[i][j].im = -fatorI/(fatorR2+fatorI2);
     	}              
    }
  }

	public void somaElementoReal(double valor){
		double 	fatorR;		// 	fator real

    for(int j = 0; j < lin; j++){
     	for(int i = 0; i < col; i++){
     		fatorR = this.MComplexa[i][j].re + valor;
     		this.MComplexa[i][j].re = fatorR;
     	}              
    }
  }

	public void somaElementoImaginario(double valor){
		double 	fatorI;		// 	fator imaginrio

    for(int j = 0; j < lin; j++){
     	for(int i = 0; i < col; i++){
     		fatorI = this.MComplexa[i][j].im + valor;
     		this.MComplexa[i][j].im = fatorI;
     	}              
    }
  }

  // Imprime a matriz com 'decimais' casas decimais
  public void imprimeMatriz(int decimais) {

		String padrao = "#.";
		for (int i = 0; i < decimais; i++)
			padrao += "#";
		DecimalFormat formato = null;
		formato = new DecimalFormat(padrao);
		String elemento;

		System.out.println();
		if (this.MComplexa == null){
   		System.out.println("Matriz vazia.");
   	} else {
   		for (int j = 0; j < lin; j++){
   			for (int i = 0; i < col; i++){
   				elemento = formato.format(this.MComplexa[i][j].re);
	 				System.out.print(elemento+ " + ");
 					elemento = formato.format(this.MComplexa[i][j].im);
 					System.out.print(elemento+ "i   ");
   			}
				System.out.println();
   		}
   	}
  }

  // Imprime a matriz com 'decimais' casas decimais
  public void imprimeReal(int decimais) {

		String padrao = "#.";
		for (int i = 0; i < decimais; i++)
			padrao += "#";
		DecimalFormat formato = null;
		formato = new DecimalFormat(padrao);
		String elemento;

		System.out.println();
		if (this.MComplexa == null){
   		System.out.println("Matriz vazia.");
   	} else {
   		for (int j = 0; j < lin; j++){
   			for (int i = 0; i < col; i++){
   				elemento = formato.format(this.MComplexa[i][j].re);
   				System.out.print(elemento+ "\t");
   			}
				System.out.println();
   		}
   	}
  }

  // Imprime a matriz com 'decimais' casas decimais
  public void imprimeImaginario(int decimais) {

		String padrao = "#.";
		for (int i = 0; i < decimais; i++)
			padrao += "#";
		DecimalFormat formato = null;
		formato = new DecimalFormat(padrao);
		String elemento;

		System.out.println();
		if (this.MComplexa == null){
   		System.out.println("Matriz vazia.");
   	} else {
   		for (int j = 0; j < lin; j++){
   			for (int i = 0; i < col; i++){
   				elemento = formato.format(this.MComplexa[i][j].im);
   				System.out.print(elemento+ "i\t");
   			}
				System.out.println();
   		}
   	}
  }

	// Gradiente: parmetros
	// dir = 1 ==> direo norte
	// dir = 2 ==> direo sul
	// dir = 3 ==> direo leste
	// dir = 4 ==> direo oeste
	public void gradiente(int dir){
		double 	pReA, // pixel real atual
						pReG, // pixel real na direo do gradiente
						pImA, // pixel imaginrio atual
						pImG; // pixel imaginrio na direo do gradiente
		
		switch(dir){
			case 1: // direo norte
				for(int i = 0; i < col; i++){
					for(int j = lin - 1; j > 0; j--){ 
						pReA = this.MComplexa[i][j].re;
						pReG = this.MComplexa[i][j-1].re;
						pImA = this.MComplexa[i][j].im;
						pImG = this.MComplexa[i][j-1].im;
						this.MComplexa[i][j].re = pReG - pReA;
						this.MComplexa[i][j].im = pImG - pImA;
					}              
				}
				for(int i = 0; i < col; i++){
					this.MComplexa[i][0].re = 0.0;
					this.MComplexa[i][0].im = 0.0;
				}
				break;
			case 2: // direo sul
				for(int i = 0; i < col; i++){
					for(int j = 0; j < lin - 1; j++){ 
						pReA = this.MComplexa[i][j].re;
						pReG = this.MComplexa[i][j+1].re;
						pImA = this.MComplexa[i][j].im;
						pImG = this.MComplexa[i][j+1].im;
						this.MComplexa[i][j].re = pReG - pReA;
						this.MComplexa[i][j].im = pImG - pImA;
					}              
				}
				for(int i = 0; i < col; i++){
					this.MComplexa[i][lin-1].re = 0.0;
					this.MComplexa[i][lin-1].im = 0.0;
				}
				break;
			case 3: // direo leste
				for(int j = 0; j < lin; j++){
					for(int i = 0; i < col - 1; i++){ 
						pReA = this.MComplexa[i][j].re;
						pReG = this.MComplexa[i+1][j].re;
						pImA = this.MComplexa[i][j].im;
						pImG = this.MComplexa[i+1][j].im;
						this.MComplexa[i][j].re = pReG - pReA;
						this.MComplexa[i][j].im = pImG - pImA;
					}              
				}
				for(int j = 0; j < lin; j++){
					this.MComplexa[col-1][j].re = 0.0;
					this.MComplexa[col-1][j].im = 0.0;
				}
				break;
			case 4:  // direo oeste
				for(int j = 0; j < lin; j++){
					for(int i = col - 1; i > 0; i--){ 
						pReA = this.MComplexa[i][j].re;
						pReG = this.MComplexa[i-1][j].re;
						pImA = this.MComplexa[i][j].im;
						pImG = this.MComplexa[i-1][j].im;
						this.MComplexa[i][j].re = pReG - pReA;
						this.MComplexa[i][j].im = pImG - pImA;
					}              
				}
				for(int j = 0; j < lin; j++){
					this.MComplexa[0][j].re = 0.0;
					this.MComplexa[0][j].im = 0.0;
				}
				break;
			}
	}
	
}
